/*
 * RAMFILE - fopen/fseek/ftell/fgetc/fread/fclose replacement to read
 *           from memory buffer.
 * (c)1998 Jari Komppa
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "cfl2.h" /* RAMfopencfl only */

typedef struct ramfilestruct {
  char * buffer;
  int offset;
  int length;
  int _flag;
} RAMFILE;

RAMFILE * RAMfopen(char * buffer, int length)
{
  RAMFILE * t;
  if (buffer==NULL) return NULL;
  t=malloc(sizeof(RAMFILE));
  t->buffer=buffer;
  t->offset=0;
  t->length=length;
  t->_flag=0;
  return t;
}

RAMFILE * RAMfopencfl(char * fname)
{
  RAMFILE * t;
  if (!CFL_IsFile(fname)) return NULL;
  t=malloc(sizeof(RAMFILE));
  t->buffer=CFL_GetFile(fname);
  t->offset=0;
  t->length=CFL_GetFileSize(fname);
  t->_flag=0;
  return t;
}

int RAMfseek(RAMFILE * t, int displacement, int mode)
{
  if (t==NULL) return 1;
  t->_flag=0;
  switch (mode) {
  case SEEK_SET: t->offset=displacement; break;
  case SEEK_END: t->offset=t->length+displacement; break;
  case SEEK_CUR: t->offset+=displacement; break;
  }
  if (t->offset>t->length) {
    t->offset=t->length;
    t->_flag=0xffffffff;
  }
  if (t->offset<0) {
    t->offset=0;
    t->_flag=0xffffffff;
  }
  return 0;
}

int RAMftell(RAMFILE * t)
{
  if (t==NULL) return -1;
  return t->offset;
}

int RAMfgetc(RAMFILE * t)
{
  if (t==NULL) return EOF;
  t->_flag=0;
  if (t->offset>=t->length) return EOF;
  t->offset++;
  if (t->offset>t->length) {
    t->offset=t->length;
    t->_flag=0xffffffff;
  }
  return (*(t->buffer+t->offset-1));
}

int RAMfread(void * buf, int size, int count, RAMFILE * t)
{
  int l;
  if (t==NULL)
    return 0;
  t->_flag=0;
  l=size*count;
  if ((l+t->offset)>t->length)
    l=t->length-t->offset;
  memcpy((char*)buf,t->buffer+t->offset,l);
  t->offset+=l;
  return l/size;
}

int RAMfclose(RAMFILE * t)
{
  if (t==NULL) return EOF;
  free(t->buffer);
  free(t);
  return 0;
}

int RAMfclose_nofree(RAMFILE * t)
{
  if (t==NULL) return EOF;
  free(t);
  return 0;
}
